@@ -36,7 +36,7 @@ FACTORIAL_LOOKUP = np.array([math.factorial(x) for x in range(20)])
 MIN_PROB = 1e-100
 
 
-#@njit
+@njit
 def fast_factorial(x):
     if x < len(FACTORIAL_LOOKUP):
         return FACTORIAL_LOOKUP[x]
@@ -48,7 +48,7 @@ def fast_factorial(x):
         return y
 
 
-#@njit
+@njit
 def poisson(rate, n, dt):
     '''Gives probability of each neurons spike count assuming poisson spiking
     '''
@@ -58,12 +58,12 @@ def poisson(rate, n, dt):
     tmp = tmp - rate*dt
     return np.exp(tmp)
 
-#@njit
+@njit
 def log_emission(rate, n , dt):
     return np.sum(np.log(poisson(rate, n, dt)))
 
 
-#@njit
+@njit
 def fix_arrays(PI,A,B):
     '''copy and remove zero values so that log probabilities can be computed
     '''
@@ -89,7 +89,7 @@ def fix_arrays(PI,A,B):
     return PI, A, B
 
 
-#@njit
+@njit
 def forward(spikes, dt, PI, A, B):
     '''Run forward algorithm to compute alpha = P(Xt = i| o1...ot, pi)
     Gives the probabilities of being in a specific state at each time point
@@ -150,7 +150,7 @@ def forward(spikes, dt, PI, A, B):
     return alpha, norms
 
 
-#@njit
+@njit
 def backward(spikes, dt, A, B, norms):
     ''' Runs the backward algorithm to compute beta = P(ot+1...oT | Xt=s)
     Computes the probability of observing all future observations given the
@@ -186,7 +186,7 @@ def backward(spikes, dt, A, B, norms):
     return beta
 
 
-#@njit
+@njit
 def compute_baum_welch(spikes, dt, A, B, alpha, beta):
     _, A, B = fix_arrays(np.array([0]), A, B)
     nTimeSteps = spikes.shape[1]
@@ -210,7 +210,7 @@ def compute_baum_welch(spikes, dt, A, B, alpha, beta):
     return gamma, epsilons
 
 
-#@njit
+@njit
 def baum_welch(trial_dat, dt, PI, A, B):
     alpha, norms = forward(trial_dat, dt, PI, A, B)
     beta = backward(trial_dat, dt, A, B, norms)
@@ -393,7 +393,7 @@ def compute_best_paths(spikes, dt, PI, A, B):
     return bestPaths, pathProbs
 
 
-#@njit
+@njit
 def compute_rate_rmse(rates1, rates2):
     # Compute RMSE per trial
     # Mean over trials
@@ -449,7 +449,7 @@ def match_states(emission1, emission2):
 
 
 @memory.cache
-#@njit
+@njit
 def convert_spikes_to_rates(spikes, dt, win_size, step_size=None):
     if step_size is None:
         step_size = win_size
@@ -466,7 +466,7 @@ def convert_spikes_to_rates(spikes, dt, win_size, step_size=None):
 
 
 @memory.cache
-#@njit
+@njit
 def generate_rate_array_from_state_seq(bestPaths, B, dt, win_size,
                                        step_size=None):
     if not step_size:
@@ -490,7 +490,7 @@ def generate_rate_array_from_state_seq(bestPaths, B, dt, win_size,
 
 
 @memory.cache
-#@njit
+@njit
 def rebin_spike_array(spikes, dt, time, new_dt):
     if dt == new_dt:
         return spikes, time
@@ -1537,3 +1537,90 @@ class ConstrainedHMM(PoissonHMM):
 
     def get_late_states(self):
         return np.arange(self.n_baseline+1, self.n_states, 2)
+
+
+def package_project_data(hmm_df, save_file, **kwargs):
+    '''packages project info, spike trains and selected hmms into a single hdf5 store
+
+    Parameters
+    ----------
+    hmm_df: pandas.DataFrame
+        dataframe with references to all hmms to store in hdf5
+        must have columns: rec_dir, exp_name, channel, hmm_id
+        all additional columns will also be stored, it is recommended to have
+        columns corresponding to hmm fitting parameters
+        all hmms stored will have their IDs changed to their index in the
+        dataframe so that hmm_ids are unique
+    save_file: str, file to save data to
+    kwargs: any additional dataframes provided will be stored with keys provided
+    '''
+    if os.path.isfile(save_file):
+        os.remove(save_file)
+
+    df = hmm_df.reset_index(drop=True)
+    with tables.open_file(save_file, 'a') as hf5:
+        for i, row in df.iterrows():
+            rd = row['rec_dir']
+            hmm_id = row['hmm_id']
+            channel = row['channel']
+            exp_name = row['exp_name']
+            dat = load_dataset(rd)
+            data_name = dat.data_name
+            din = dat.dig_in_mapping.set_index('channel').loc[channel, 'name']
+            din = din.replace(' ', '_')
+            hmm_h5 = hmmIO.get_hmm_h5(rd)
+            PI, A, B, stat_arrays, params = hmmIO.read_hmm_from_hdf5(hmm_h5, hmm_id)
+
+            path_str = '/'
+            if exp_name not in hf5.root:
+                hf5.create_group('/', exp_name, exp_name + ' Data and Models')
+
+            path_str += exp_name
+            if data_name not in hf5.root[exp_name]:
+                hf5.create_group(path_str, data_name, 'Single recording data')
+
+            path_str += '/' + data_name
+            if din not in hf5.root[path_str]:
+                hf5.create_group(path_str, din, 'data around digital input event')
+
+            path_str += '/' + din
+
+            # Store data
+            if 'spikes' not in hf5.root[path_str]:
+                spikes, dt, spike_time = get_hmm_spike_data(rd,
+                                                            params['unit_type'],
+                                                            channel,
+                                                            time_start=params['time_start'],
+                                                            time_end=params['time_end'],
+                                                            area=params['area'])
+                hf5.create_array(path_str, 'spikes', spikes)
+                hf5.create_array(path_str, 'spike_time', spike_time)
+
+
+            h_str = 'hmm_%i' % i
+            hf5.create_group(path_str, h_str, 'Fitted Hidden Markov Model Parameters')
+            path_str += '/' + h_str
+            hf5.create_group(path_str, 'model_params', 'HMM model parameters')
+            p_str = path_str + '/model_params'
+            hf5.create_group(path_str, 'model_data', 'outputs of the model')
+            d_str = path_str + '/model_data'
+
+            hf5.create_array(d_str, 'hmm_time', stat_arrays['time'])
+            hf5.create_array(d_str, 'state_sequences', stat_arrays['best_sequences'])
+            hf5.create_array(d_str, 'state_probabilities',
+                             stat_arrays['gamma_probabilities'])
+            hf5.create_array(d_str, 'row_id', stat_arrays['row_id'])
+
+            hf5.create_array(p_str, 'initial_probabilties', PI)
+            hf5.create_array(p_str, 'transition', A)
+            hf5.create_array(p_str, 'emission', B)
+            hf5.flush()
+
+        df['hmm_id'] = df.index
+        df.to_hdf(save_file, 'hmm_overview')
+        for k,v in kwargs.items():
+            if isinstance(v, pd.DataFrame):
+                v.to_hdf(save_file, k)
+            else:
+                print(f'Invalid datatype for {k}. Must be pandas.DataFrame. Not written to HDF5') 
+
